/*********************************************************
*Copyright (C), 2021, Shanghai Eastsoft Microelectronics Co., Ltd.
*文件名:  main.c
*作  者:  AE Team
*版  本:  V1.00
*日  期:  2022/1/28
*描  述:  FreeRTOS例程，printf打印，TX:PA8
          可调用标准库使用，需要以下三个步骤
          1、keil勾选Use MicroLIB ，2、main.c #include "stdio.h"，3、 #define __PRINTF_USE_UART3__
          2、IAR的Library选择Full
*备  注:  适用于ES8H018x芯片
          本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include <stdio.h>
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <event_groups.h>
#include <stream_buffer.h>
#include <semphr.h>
#include "main.h"

/*任务句柄*/
static TaskHandle_t g_task_app_handle = NULL;
static TaskHandle_t g_task_1_handle = NULL;
static TaskHandle_t g_task_2_handle = NULL;
static SemaphoreHandle_t g_sema_test = NULL;

/* 任务 */
static void task_app(void *arg);
static void task_1(void *arg);
static void task_2(void *arg);

int main(void)
{
    int retval;
    SystemHRCSelect(SCU_HRC_16M);       //HRC选择16MHz
    SystemClockConfig();                //配置时钟HRC
    DeviceClockAllEnable();             //打开所有外设时钟

    UARTInit();                         //UART初始化(115200,8,NO,1)
    printf("Startup\r\n");

    retval = xTaskCreate(task_app, "AppTaskCreate", 128, NULL, 1, &g_task_app_handle);

    /* Start task schedule*/
    if (pdPASS == retval)
        vTaskStartScheduler();
    else
        return -1;

    /* Should nerver run to here */
    while (1)
    {
    }
}

static void task_app(void *arg)
{
    BaseType_t retval = pdPASS;

    taskENTER_CRITICAL();

    /* Create semaphore */
    g_sema_test = xSemaphoreCreateBinary();

    if (g_sema_test != NULL)
        printf("Create Test_Queue OK\r\n");

    xSemaphoreGive(g_sema_test);

    /* Create task 1 */
    retval = xTaskCreate(task_1, "Task_1", 128, NULL, 4, &g_task_1_handle);

    if (pdPASS == retval)
        printf("Task 1 Created\r\n");

    /* Create task 2 */
    retval = xTaskCreate(task_2, "Task_2", 128, NULL, 3, &g_task_2_handle);

    if (pdPASS == retval)
        printf("Task 2 Created\r\n");

    /* Delete self */
    vTaskDelete(g_task_app_handle);

    taskEXIT_CRITICAL();
}

static void task_1(void *arg)
{
    for (;;)
    {
        vTaskDelay(1000);
        printf("This is task 1\r\n");
    }
}

static void task_2(void *arg)
{
    for (;;)
    {
        /* Try to obtain the semaphore */
        if (xSemaphoreTake(g_sema_test, 1000) == pdTRUE)
        {
            printf("Task2 get Semaphore\r\n");
            vTaskDelay(1000);
        }
        else
        {
            printf("Task2 fai to get Semaphore\r\n");
        }

    }
}
void callback_from_isr()
{
    BaseType_t taskWoken;

    if (xSemaphoreGiveFromISR(g_sema_test, &taskWoken) == pdTRUE)
    {
        portEND_SWITCHING_ISR(taskWoken);
    }
}
