/*********************************************************
*Copyright (C), 2021, Shanghai Eastsoft Microelectronics Co., Ltd.
*文件名:  main.c
*作  者:  AE Team
*版  本:  V1.00
*日  期:  2022/1/28
*描  述:  FreeRTOS例程，printf打印，TX:PA8
          可调用标准库使用，需要以下三个步骤
          1、keil勾选Use MicroLIB ，2、main.c #include "stdio.h"，3、 #define __PRINTF_USE_UART3__
          2、IAR的Library选择Full
*备  注:  适用于ES8H018x芯片
          本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include <stdio.h>
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <event_groups.h>
#include <stream_buffer.h>
#include <semphr.h>
#include "main.h"

#define BIT_ALL 0xFFFFFFFF
#define BIT(x) (1<<(x))

/*任务句柄*/
static TaskHandle_t g_task_app_handle = NULL;
static TaskHandle_t g_task_1_handle = NULL;
static TaskHandle_t g_task_2_handle = NULL;
static TaskHandle_t g_task_3_handle = NULL;

/* 任务 */
static void task_app(void *arg);
static void task_1(void *arg);
static void task_2(void *arg);
static void task_3(void *arg);

int main(void)
{
    int retval;
    SystemHRCSelect(SCU_HRC_16M);       //HRC选择16MHz
    SystemClockConfig();                //配置时钟HRC
    DeviceClockAllEnable();             //打开所有外设时钟

    UARTInit();                         //UART初始化(115200,8,NO,1)
    printf("Startup\r\n");

    retval = xTaskCreate(task_app, "AppTaskCreate", 128, NULL, 1, &g_task_app_handle);

    /* Start task schedule*/
    if (pdPASS == retval)
        vTaskStartScheduler();
    else
        return -1;

    /* Should nerver run to here */
    while (1)
    {
    }
}

static void task_app(void *pvParameters)
{
    BaseType_t retval = pdPASS;

    taskENTER_CRITICAL();

    /* Create task 1 */
    retval = xTaskCreate(task_1, "Task_1", 128, NULL, 2, &g_task_1_handle);

    if (pdPASS == retval)
        printf("Task 1 Created\r\n");

    /* Create task 2 */
    retval = xTaskCreate(task_2, "Task_2", 128, NULL, 3, &g_task_2_handle);

    if (pdPASS == retval)
        printf("Task 2 Created\r\n");

    /* Create notify generator task*/
    retval = xTaskCreate(task_3, "Notify_Task", 128, NULL, 4, &g_task_3_handle);

    if (pdPASS == retval)
        printf("Task Notify Gen Created\r\n");

    /* Delete self */
    vTaskDelete(g_task_app_handle);

    taskEXIT_CRITICAL();
}

static void task_1(void *pvParameters)
{
    uint32_t val = 0U;

    while (1)
    {
        /* Waiting for notify bit1=0x02 */
        if (xTaskNotifyWait(0, BIT_ALL, &val, portMAX_DELAY) == pdTRUE)
        {
            if ((val & BIT(1)) == BIT(1))
            {
                printf("Task1 receive notify %d\r\n", val);
            }
        }
    }
}

static void task_2(void *pvParameters)
{
    uint32_t val = 0U;

    for (;;)
    {
        if (xTaskNotifyWait(0, BIT_ALL, &val, portMAX_DELAY) == pdTRUE)
        {
            /* Waiting for notify bit0 and bit2 = 0x05 */
            if ((val & (BIT(0) | BIT(2))) == (BIT(0) | BIT(2)))
            {
                printf("Task2 receive notify %d\r\n", val);
            }
        }
    }
}

static void task_3(void *pvParameters)
{
    uint32_t val = 0U;
    uint32_t i = 0U;

    for (;;)
    {
        /* Generate notify sequence bit0 bit01 bit012 */
        for (i = 0; i < 3; i++)
        {
            val |= BIT(i);
            xTaskNotify(g_task_1_handle, val, eSetBits);
            xTaskNotify(g_task_2_handle, val, eSetBits);
            printf("Signal gen %u\r\n", val);

            vTaskDelay(1000);
        }

        val = 0;
    }
}
